// (c) K1EL Steven T. Elliott
// modifiziert dk2jk 2020

#include "tiny_tone.h"

const int speed_Pot = A1;  // CW Speed Pot Pin
const int key_dit = 3;  // Keyer Dit input pin
const int key_dah = 4;  // Keyer Dah input pin
const int key_pin = 0;  // Key Out pin
const int key_mode = 0; // Set to 0 for Iambic A mode. Set to 1 for Iambic B, Set to 2 for Straight Key
const int tone_pin = 1;
const int tonfrequenz = 650;
// Copyright (c) 2009 Steven T. Elliott
// modifiziert: dk2jk 2020

TinyTone mithoerton ( tone_pin, tonfrequenz);

#define SPEED_MAX 30
#define SPEED_MIN 10
int cw_speed = SPEED_MIN;
int old_speed =SPEED_MAX;

// Keyer Variables
unsigned long ditTime;  // Number of milliseconds per dit
char keyerControl;
char keyerState;
static unsigned long ktimer;

/////////////////////////////////////////////////////////////////////////////////////////
//  keyerControl bit definitions
//
#define     DIT_L      0x01     // Dit latch
#define     DAH_L      0x02     // Dah latch
#define     DIT_PROC   0x04     // Dit is being processed
#define     PDLSWAP    0x08     // 0 for normal, 1 for swap
#define     IAMBICB    0x10     // 0 for Iambic A, 1 for Iambic B
#define     ULTIMATIC  0x20     // 1 for ultimatic
#define     STRAIGHT   0x80     // 1 for straight key mode 


////////////////////////////////////////////////////////////////////////////////////////
//
//  State Machine Defines

enum KSTYPE {IDLE, CHK_DIT, CHK_DAH, KEYED_PREP, KEYED, INTER_ELEMENT };

void setup()
{
  // Set up the Keyer
  pinMode (key_dit, INPUT);
  pinMode (key_dah, INPUT);
  pinMode (key_pin, OUTPUT);
  // Enable the Internal Pull-up resistor on the Keyer Inputs
  digitalWrite(key_dit, HIGH);
  digitalWrite(key_dah, HIGH);

  // Setup the Keyer
  keyerState = IDLE;
  keyerControl = 0;
}  // End Setup

void loop()
{
  if (keyerState != KEYED)
  { set_keyer_speed(); // Read and set the Keyer Speed
  }
  check_key();  // Check the Key and Transmit CW
}  // End Main Loop


///////////////////////////////////////////////////////////////////////////////
//
//    Latch dit and/or dah press
//
//    Called by keyer routine
//
///////////////////////////////////////////////////////////////////////////////

void update_PaddleLatch()
{ if (digitalRead(key_dit) == LOW) {
    keyerControl |= DIT_L;
  }
  if (digitalRead(key_dah) == LOW) {
    keyerControl |= DAH_L;
  }
}

///////////////////////////////////////////////////////////////////////////////
//
//    Calculate new time constants based on wpm value
//
///////////////////////////////////////////////////////////////////////////////

void loadWPM (int wpm)
{ ditTime = 1200 / wpm;
}


void set_keyer_speed()
{ if (key_mode != 2)
  { // Read and set the CW Keyer Speed
    unsigned int x = analogRead(speed_Pot); // Read the CW Speed Pot
    x = map(x, 10, 1024 , SPEED_MIN, SPEED_MAX); // umrechnen
    cw_speed = constrain(x, SPEED_MIN, SPEED_MAX); // gegrenzen
    if (cw_speed != old_speed)  // Update if the speed has changed
    { old_speed = cw_speed;
      loadWPM(cw_speed);   // Set the keying speed
    }
  }
}

void check_key()
{ if (key_mode == 2) // Straight Key
  { // Straight Key Mode
    if ((digitalRead(key_dit) == LOW) || (digitalRead(key_dah) == LOW))
    { keyerState = KEYED;
      // Key from either paddle
      digitalWrite(key_pin, HIGH);
    } else {
      digitalWrite(key_pin, LOW);
      keyerState = IDLE;
    }
  } else {
    // Basic Iambic Keyer
    // keyerControl contains processing flags and keyer mode bits
    // Supports Iambic A and B
    // State machine based, uses calls to millis() for timing.
    switch (keyerState)
    { case IDLE:      // Wait for direct or latched paddle press
        if ((digitalRead(key_dah) == LOW) || (digitalRead(key_dit) == LOW) || (keyerControl & 0x03))
        { update_PaddleLatch();
          keyerState = CHK_DIT;
        }
        break;
      case CHK_DIT:      // See if the dit paddle was pressed
        if (keyerControl & DIT_L)
        { keyerControl |= DIT_PROC;
          ktimer = ditTime;
          keyerState = KEYED_PREP;
        } else {
          keyerState = CHK_DAH;
        }
        break;
      case CHK_DAH:      // See if dah paddle was pressed
        if (keyerControl & DAH_L)
        { ktimer = ditTime * 3;
          keyerState = KEYED_PREP;
        } else {
          keyerState = IDLE;
        }
        break;
      case KEYED_PREP:      // Assert key down, start timing, state shared for dit or dah
        digitalWrite(key_pin, HIGH);         // Key the TX Relay
        ktimer += millis();                  // set ktimer to interval end time
        keyerControl &= ~(DIT_L + DAH_L);    // clear both paddle latch bits
        keyerState = KEYED;                  // next state
        mithoerton.on();
        break;
      case KEYED:      // Wait for timer to expire

        if (millis() > ktimer)  // are we at end of key down ?
        { digitalWrite(key_pin, LOW);      // Turn the TX Relay off
          ktimer = millis() + ditTime;     // inter-element time
          keyerState = INTER_ELEMENT;      // next state
          mithoerton.off();
        }
        else if (key_mode == 1) // Check to see if we're in Iambic B Mode
        { update_PaddleLatch();           // early paddle latch in Iambic B mode
        }
        break;
        break;
      case INTER_ELEMENT:      // Insert time between dits/dahs
        update_PaddleLatch();               // latch paddle state
        if (millis() > ktimer)  // are we at end of inter-space ?
        { if (keyerControl & DIT_PROC)      // was it a dit or dah ?
          { keyerControl &= ~(DIT_L + DIT_PROC);   // clear two bits
            keyerState = CHK_DAH;                  // dit done, check for dah
          } else {
            keyerControl &= ~(DAH_L);              // clear dah latch
            keyerState = IDLE;                     // go idle
          }
        }
        break;
    }
  }
}
